@@ -12,4 +12,6 @@ if statsd is None:
     port = int(os.getenv('STATSD_PORT', defaults.PORT))
     prefix = os.getenv('STATSD_PREFIX', defaults.PREFIX)
     maxudpsize = int(os.getenv('STATSD_MAXUDPSIZE', defaults.MAXUDPSIZE))
-    statsd = StatsClient(host, port, prefix, maxudpsize)
+    ipv6 = bool(int(os.getenv('STATSD_IPV6', defaults.IPV6)))
+    statsd = StatsClient(host=host, port=port, prefix=prefix,
+                         maxudpsize=maxudpsize, ipv6=ipv6)
